import { render } from '@testing-library/react';
import TiledLayer from './TiledLayer';
import { TileFactory } from './types';

const fakeTileFactory: TileFactory = (keyObject) => (
  <text>{JSON.stringify(keyObject)}</text>
);

describe('The TiledLayer component ', () => {
  test('exposes the tiles needed per its viewport', () => {
    const { baseElement } = render(
      <svg>
        <TiledLayer
          keyObject={{ provider: 'osm', zoomLevel: 10, x: 5, y: 8 }}
          viewPort={{ topLeft: { x: 1, y: 2 }, bottomRight: { x: 3, y: 2 } }}
          tileFactory={fakeTileFactory}
        />
      </svg>
    );
    //    screen.debug();
    expect(baseElement).toMatchInlineSnapshot(`
<body>
  <div>
    <svg>
      <g
        transform="translate(1, 2)"
      >
        <text>
          {"provider":"osm","zoomLevel":10,"x":6,"y":10}
        </text>
      </g>
      <g
        transform="translate(2, 2)"
      >
        <text>
          {"provider":"osm","zoomLevel":10,"x":7,"y":10}
        </text>
      </g>
      <g
        transform="translate(3, 2)"
      >
        <text>
          {"provider":"osm","zoomLevel":10,"x":8,"y":10}
        </text>
      </g>
    </svg>
  </div>
</body>
`);
  });

  test('adds more tiles when its viewport is updated without removing the previous ones', () => {
    const { baseElement, rerender } = render(
      <svg>
        <TiledLayer
          keyObject={{ provider: 'osm', zoomLevel: 10, x: 5, y: 8 }}
          viewPort={{ topLeft: { x: 1, y: 2 }, bottomRight: { x: 3, y: 2 } }}
          tileFactory={fakeTileFactory}
        />
      </svg>
    );
    rerender(
      <svg>
        <TiledLayer
          keyObject={{ provider: 'osm', zoomLevel: 10, x: 5, y: 8 }}
          viewPort={{ topLeft: { x: 5, y: 0 }, bottomRight: { x: 5, y: 2 } }}
          tileFactory={fakeTileFactory}
        />
      </svg>
    );
    //    screen.debug();
    expect(baseElement).toMatchInlineSnapshot(`
<body>
  <div>
    <svg>
      <g
        transform="translate(1, 2)"
      >
        <text>
          {"provider":"osm","zoomLevel":10,"x":6,"y":10}
        </text>
      </g>
      <g
        transform="translate(2, 2)"
      >
        <text>
          {"provider":"osm","zoomLevel":10,"x":7,"y":10}
        </text>
      </g>
      <g
        transform="translate(3, 2)"
      >
        <text>
          {"provider":"osm","zoomLevel":10,"x":8,"y":10}
        </text>
      </g>
      <g
        transform="translate(5, 0)"
      >
        <text>
          {"provider":"osm","zoomLevel":10,"x":10,"y":8}
        </text>
      </g>
      <g
        transform="translate(5, 1)"
      >
        <text>
          {"provider":"osm","zoomLevel":10,"x":10,"y":9}
        </text>
      </g>
      <g
        transform="translate(5, 2)"
      >
        <text>
          {"provider":"osm","zoomLevel":10,"x":10,"y":10}
        </text>
      </g>
    </svg>
  </div>
</body>
`);
  });
  test('is reinitialized if its key isObject updated', () => {
    const { baseElement, rerender } = render(
      <svg>
        <TiledLayer
          keyObject={{ provider: 'osm', zoomLevel: 10, x: 5, y: 8 }}
          viewPort={{ topLeft: { x: 1, y: 2 }, bottomRight: { x: 3, y: 2 } }}
          tileFactory={fakeTileFactory}
        />
      </svg>
    );
    rerender(
      <svg>
        <TiledLayer
          keyObject={{ provider: 'osm', zoomLevel: 11, x: 5, y: 8 }}
          viewPort={{ topLeft: { x: 5, y: 0 }, bottomRight: { x: 5, y: 2 } }}
          tileFactory={fakeTileFactory}
        />
      </svg>
    );
    //    screen.debug();
    expect(baseElement).toMatchInlineSnapshot(`
<body>
  <div>
    <svg>
      <g
        transform="translate(5, 0)"
      >
        <text>
          {"provider":"osm","zoomLevel":11,"x":10,"y":8}
        </text>
      </g>
      <g
        transform="translate(5, 1)"
      >
        <text>
          {"provider":"osm","zoomLevel":11,"x":10,"y":9}
        </text>
      </g>
      <g
        transform="translate(5, 2)"
      >
        <text>
          {"provider":"osm","zoomLevel":11,"x":10,"y":10}
        </text>
      </g>
    </svg>
  </div>
</body>
`);
  });
  test('Also works with negative coordinates', () => {
    const { baseElement } = render(
      <svg>
        <TiledLayer
          keyObject={{ provider: 'osm', zoomLevel: 10, x: 5, y: 8 }}
          viewPort={{ topLeft: { x: -3, y: -1 }, bottomRight: { x: -2, y: 2 } }}
          tileFactory={fakeTileFactory}
        />
      </svg>
    );
    //    screen.debug();
    expect(baseElement).toMatchInlineSnapshot(`
<body>
  <div>
    <svg>
      <g
        transform="translate(-3, -1)"
      >
        <text>
          {"provider":"osm","zoomLevel":10,"x":2,"y":7}
        </text>
      </g>
      <g
        transform="translate(-2, -1)"
      >
        <text>
          {"provider":"osm","zoomLevel":10,"x":3,"y":7}
        </text>
      </g>
      <g
        transform="translate(-3, 0)"
      >
        <text>
          {"provider":"osm","zoomLevel":10,"x":2,"y":8}
        </text>
      </g>
      <g
        transform="translate(-2, 0)"
      >
        <text>
          {"provider":"osm","zoomLevel":10,"x":3,"y":8}
        </text>
      </g>
      <g
        transform="translate(-3, 1)"
      >
        <text>
          {"provider":"osm","zoomLevel":10,"x":2,"y":9}
        </text>
      </g>
      <g
        transform="translate(-2, 1)"
      >
        <text>
          {"provider":"osm","zoomLevel":10,"x":3,"y":9}
        </text>
      </g>
      <g
        transform="translate(-3, 2)"
      >
        <text>
          {"provider":"osm","zoomLevel":10,"x":2,"y":10}
        </text>
      </g>
      <g
        transform="translate(-2, 2)"
      >
        <text>
          {"provider":"osm","zoomLevel":10,"x":3,"y":10}
        </text>
      </g>
    </svg>
  </div>
</body>
`);
  });

});
